\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("terdragon-defines.gp")

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ hull limit side lengths

core_of_frac(f) = core(f*denominator(f)^2);

{
  my(v = hull(10));
  v = hull_limit;
  for(i=0,#v-1,
     my(p1 = vector_modulo(v,i),
        p2 = vector_modulo(v,i+1),
       n = norm(p1-p2),
       sqfree = core_of_frac(n)
       );
     print(n*24^2,"   issquare ",issquare(n), " sqfree ",sqfree));
}


\\-----------------------------------------------------------------------------

polygon_boundary(v) =
{
  my(total = 0);
  for(i=1,#v,
     my(p1 = vector_modulo(v,i),
        p2 = vector_modulo(v,i+1));
     total += abs(p1-p2));
  total;
}

\\-----------------------------------------------------------------------------

P1_to_P2(k) = norm(P1(k)-P2(k));
P2_to_P3(k) = norm(P2(k)-P3(k));
P3_to_P4(k) = norm(P3(k)-P4(k));
P4_to_P5(k) = norm(P4(k)-P5(k));
P5_to_P6(k) = norm(P5(k)-P6(k));
P6_to_P7(k) = norm(P6(k)-P7(k));
P7_to_P1dash(k) = norm(P7(k)-P1dash(k));

{
  my(k=20);
  check_exact(nearly_equal(sqrt(P1_to_P2(k)),
                           abs(P1(k)-P2(k))),
              1);
}

\\ whole
     \\ + -4/9 - 1/3*x - x^4
     \\ + (7/32)/(1 - x)
     \\ + (-1/64)/(1 + x)
     \\ + (3/16 + 3/16*x)/(1 + x^2)
     \\ + (1/48 - 1/16*x)/(1 - 3*x^2)
     \\ + (1/32 - 1/32*x)/(1 + 3*x^2)
     \\ + (1/576)/(1 - 3*x)

gP1_to_P2_sqrt(k) = {
     + -2/3 + 1/3*x - x^4
     + (1/4)/(1 + x)
     + (3/8 - 1/8*x)/(1 + x^2)
     + (1/24 + 1/24*x)/(1 - 3*x^2);
}
gf_terms(     + (1/4)/(1 + x)      + (3/8 - 1/8*x)/(1 + x^2), 8)

P1_to_P2_sqrt(k) =
{
  1/24*3^floor(k/2) + 1/8*vector_modulo([5, -3, -1, -1],k);
}
check_exact(gf_terms(gP1_to_P2_sqrt(x), 100), \
            vector(100,k,k--; if(k<6,0, P1_to_P2_sqrt(k))));

P1_to_P7(k) =
{
  sqrt_as_quad(P1_to_P2(k))
  + sqrt_as_quad(P2_to_P3(k))
  + sqrt_as_quad(P3_to_P4(k))
  + sqrt_as_quad(P4_to_P5(k))
  + sqrt_as_quad(P5_to_P6(k))
  + sqrt_as_quad(P6_to_P7(k));
}
P1_to_P7_inexact(k) =
{
  sqrt(P1_to_P2(k))
  + sqrt(P2_to_P3(k))
  + sqrt(P3_to_P4(k))
  + sqrt(P4_to_P5(k))
  + sqrt(P5_to_P6(k))
  + sqrt(P6_to_P7(k));
}

\\ print("P1_to_P7");
\\ vector(100,k,k--;if(k<6,0, P1_to_P7(k)))


P1_to_P7_powers(k) =
{
  (13 + 5*sqrt3)/24 * sqrt3^k
  + 1/8 * vector_modulo([9 - 7*sqrt3, 3 - 7*sqrt3, 3 - 5*sqrt3, 9 - 5*sqrt3],k)
  ;
}
check_exact(vector(100,k,k--; if(k<6,0, P1_to_P7_powers(k))), \
            vector(100,k,k--; if(k<6,0, P1_to_P7(k))));

P1_to_P7_powers(k) =
{
  1/24* vector_modulo([13 + 5*sqrt3,15 + 13*sqrt3],k) * 3^floor(k/2) 
  + 1/8 * vector_modulo([9 - 7*sqrt3, 3 - 7*sqrt3, 3 - 5*sqrt3, 9 - 5*sqrt3],k)
  ;
}
check_exact(vector(100,k,k--; if(k<6,0, P1_to_P7_powers(k))), \
            vector(100,k,k--; if(k<6,0, P1_to_P7(k))));

P7_to_P1dash_norm_powers(k) =
{
  37/144 * 3^k
  + 1/24 * vector_modulo([-5, 27, 5, -27],k) * 3^floor(k/2)
  + 1/16 * vector_modulo([1, 7],k)
  ;
}
check_exact(vector(100,k,k--; if(k<6,0, P7_to_P1dash_norm_powers(k))), \
            vector(100,k,k--; if(k<6,0, P7_to_P1dash(k))));

hull_boundary(k) = 2*P1_to_P7(k) + 2*sqrt(P7_to_P1dash_norm_powers(k));
{
  my(k=12,v = hull(k));
  print("polygon_boundary() ",polygon_boundary(v));
  print("hull_boundary()    ",hull_boundary(k));
  print("P1_to_P7           ",P1_to_P7(k));
  print("P1_to_P7_inexact   ",P1_to_P7_inexact(k));

  \\ print("P1 to P2   ",sqrt(P1_to_P2(k)));
  \\ print("P2 to P2   ",sqrt(P2_to_P3(k)));
  \\ print("P3 to P2   ",sqrt(P3_to_P4(k)));
  \\ print("P4 to P2   ",sqrt(P4_to_P5(k)));
  \\ print("P5 to P2   ",sqrt(P5_to_P6(k)));
  \\ print("P6 to P2   ",sqrt(P6_to_P7(k)));
  \\ print("P7 to P1'  ",sqrt(P7_to_P1dash(k)));
  \\ 
  \\ print("v1 to v2   ",sqrt(norm(v[1]-v[2])));
  \\ print("v2 to v2   ",sqrt(norm(v[2]-v[3])));
  \\ print("v3 to v2   ",sqrt(norm(v[3]-v[4])));
  \\ print("v4 to v2   ",sqrt(norm(v[4]-v[5])));
  \\ print("v5 to v2   ",sqrt(norm(v[5]-v[6])));
  \\ print("v6 to v2   ",sqrt(norm(v[6]-v[7])));
  \\ print("v7 to v1'  ",sqrt(norm(v[7]-v[8])));
}
{
  my(k=12);
  check_exact(nearly_equal(polygon_boundary(hull(k)),
                           2*P1_to_P7(k)
                           + 2*sqrt(P7_to_P1dash_norm_powers(k))),
              1);
}

\\ print("P7_to_P1dash square part");
\\ vector(100,k,k--;if(k<6,0, (P7_to_P1dash(k))))
 \\ print("P7_to_P1dash sqrt part");
 \\ vector(100,k,k--;if(k<6,0, core(P7_to_P1dash(k))))
 \\ print("P7_to_P1dash unsquared part");
 \\ vector(100,k,k--;if(k<6,0, sqrtint(P7_to_P1dash(k)/core(P7_to_P1dash(k)))))


\\-----------------------------------------------------------------------------

{
check_exact(1/(x + y*B1 + z*C1),
            1/(x + y*B1 + z*C1) * (x+y*B1 - z*C1)/(x+y*B1 - z*C1))
}
{
check_exact(1/(x + y*B1 + z*C1),
            (x+y*B1 - z*C1)  /((x + y*B1)^2 - z^2*C1^2))
}
{
check_exact(1/(x + y*B1 + z*C1),
            (x+y*B1 - z*C1)  /(x^2+y^2*B1^2-z^2*C1^2 + 2*x*y*B1))
}
{
check_exact(1/(x + y*B1 + z*C1),
            (x+y*B1 - z*C1) * (x^2+y^2*B1^2-z^2*C1^2 - 2*x*y*B1)
            / ((x^2+y^2*B1^2-z^2*C1^2)^2 - (2*x*y*B1)^2))
}
{
check_exact(1/(x + y*B1 + z*C1),
            (x+y*B1 - z*C1) * (x^2 + y^2*B1^2 - z^2*C1^2 - 2*x*y*B1)
            / (x^4
               - 2*x^2*z^2 * C1^2
               + z^4 * C1^4
               + y^4 * B1^4
               - 2*x^2*y^2 * B1^2
               - 2*y^2*z^2 * B1^2 * C1^2
              ))
}

\\-----------------------------------------------------------------------------

perimeter = polygon_boundary(hull_limit);
print("perimeter ",perimeter, "  squared ",perimeter^2);

area = geom_area_of_polygon(hull_limit);
print("area ",area,"  ",area*1.0);

\\ circle
\\ r=10;
\\ perimeter = 2*Pi*r;
\\ area = Pi*r^2;

print("circularity ", 4*Pi*area/perimeter^2);

HB_limit = 2*(13/24 + 5/24*sqrt(3) + sqrt(37)/12);
print("            ", 4*Pi*area/HB_limit^2);

{
  my(x=2*13/24, y=2*5/24, \\ B1=3, C1=37,
     z=2*1/12,
     B12 = 3,
     C12 = 37);

  my(den = (x^4
               - 2*x^2*z^2 * C12
               + z^4 * C12^2
               + y^4 * B12^2
               - 2*x^2*y^2 * B12
               - 2*y^2*z^2 * B12 * C12
              ));
  print("den  ",den);

  my(num = (x+y*B1 - z*C1) * (x^2 + y^2*B12 - z^2*C12 - 2*x*y*B1));
  print("num      ",num);
  print("num      ",subst(subst(num,'B1,sqrt(3)),'C1,sqrt(37)));
  print("num/den  ",subst(subst(num,'B1,sqrt(3)),'C1,sqrt(37)) / den);

  num = -325/864*B12 + (65/432*C1 - 605/864)*B1 + (-1/9*C1 + 13/18);
  print("num      ",num);
  print("num      ",subst(subst(num,'B1,sqrt(3)),'C1,sqrt(37)));
  print("num/den  ",subst(subst(num,'B1,sqrt(3)),'C1,sqrt(37)) / den);

  num = (65/432*C1 - 605/864)*B1 + (-1/9*C1 - 13/32);
  print("num      ",num);
  print("num      ",subst(subst(num,'B1,sqrt(3)),'C1,sqrt(37)));
  print("num/den  ",subst(subst(num,'B1,sqrt(3)),'C1,sqrt(37)) / den);

  my(f = ((65/432*C1 - 605/864)*B1 + (-1/9*C1 - 13/32)) / den);
  print("f  ",f);
  print("f  ",subst(subst(f,'B1,sqrt(3)),'C1,sqrt(37)));

  f = (-260/3457*C1 + 1210/3457)*B1 + (192/3457*C1 + 702/3457);

  my(f2 = f^2);
  print("f2  ",f2);
  print("f2  ",subst(subst(f2,'B1,sqrt(3)),'C1,sqrt(37)));

  f2 = (67600/11950849*C12 - 629200/11950849*C1 + 1464100/11950849)*B12 + (-99840/11950849*C12 + 99600/11950849*C1 + 1698840/11950849)*B1 + (36864/11950849*C12 + 269568/11950849*C1 + 492804/11950849);
  print("f2  ",f2);
  print("f2  ",subst(subst(f2,'B1,sqrt(3)),'C1,sqrt(37)));

  f2 = (99600/11950849*C1 - 1995240/11950849)*B1 + (-1618032/11950849*C1 + 13752672/11950849);
  print("f2  ",f2);
  print("f2  ",subst(subst(f2,'B1,sqrt(3)),'C1,sqrt(37)));

  my(circ = 4*29/96*B1*f2);
  print("circ  ",circ);

  circ = (120350/11950849*C1 - 2410915/11950849)*B12 + (-1955122/11950849*C1 + 16617812/11950849)*B1;
  print("circ  ",circ);

  circ = (-1955122*B1*C1 + 16617812*B1 + 361050*C1 - 7232745) / 11950849;
  print("circ  ",subst(subst(circ,'B1,sqrt(3)),'C1,sqrt(37)) * Pi);

  my(B1 = sqrt(3), C1=sqrt(37));
  print("1/HB_limit       ",1/HB_limit);
  print("1/(x+y*B1+z*C1)  ",1/(x+y*B1+z*C1));

  f2 = (99600*B1*C1 - 1995240*B1 - 1618032*C1 + 13752672) / 11950849;
  print("circ  "4*Pi*area*f2);

}


quit

{
for(k=6,26,print(core(P1_to_P2(k))," ",
                 core(P2_to_P3(k))," ",
                 core(P3_to_P4(k))," ",
                 core(P4_to_P5(k))," ",
                 core(P5_to_P6(k))," ",
                 core(P6_to_P7(k))," ",
                 core(P7_to_P1dash(k))));
}

{
for(k=6,26,print((P1_to_P2(k))," ",
                 (P2_to_P3(k))," ",
                 (P3_to_P4(k))," ",
                 (P4_to_P5(k))," ",
                 (P5_to_P6(k))," ",
                 (P6_to_P7(k))," ",
                 (P7_to_P1dash(k))));
}