#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

use lib $FindBin::Bin;
use MyTerdragon 'A','turn','triangle_normalize','chamfer';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";
my $level = 3;

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\node (start) at (0,0) [my dot];\n";
print "  \\node (end) at ($x_hi,$y_hi) [my dot];\n";

sub is_touch {
  my ($n, $n_hi) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_other ($path->xy_to_n_list($x,$y)) {
    next if $n_other == $n;
    next if $n_other > $n_hi;
    return 1;
  }
  return 0;
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
my $join = '';
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my $semi = ($n == $n_hi ? ';' : '');

  if ($n == $n_lo) {
    printwrap("$join(start)");
  } elsif ($n == $n_hi) {
    printwrap("$join(end);\n");
  } elsif (is_touch($n,$n_hi)) {
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
    my $sx = $x . chamfer(-$prev_dx);
    my $sy = $y . chamfer(-$prev_dy);
    my $tx = $x . chamfer($dx);
    my $ty = $y . chamfer($dy);
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");
  } else {
    printwrap("--($x,$y)");
  }
  $join = '--';
}


my $TurnRuns2 = 0;
foreach my $want_turn (1, -1) {
  my $side   = ($want_turn == 1 ? "left" : "right");
  my $colour = ($want_turn == 1 ? "" : "[my grey]");
  $Printwrap = 2;
  printwrap("\\fill$colour foreach \\p in {");
  $Printwrap += 2;
  foreach my $n ($n_lo+1 .. $n_hi-3) {
    if (turn($n) == $want_turn
        && turn($n+1) == $want_turn) {

      my ($x1,$y1) = $path->n_to_xy($n-1);
      my ($x2,$y2) = $path->n_to_xy($n);
      my ($x3,$y3) = $path->n_to_xy($n+1);

      # if ($want_turn == -1) {  # anti-clockwise around
      #   ($x1,$y1, $x3,$y3)= ($x3,$y3, $x1,$y1);
      # }
      ### triangle: "n=$n  $x1,$y1, $x2,$y2, $x3,$y3  want_turn=$want_turn"

      ($x1,$y1, $x2,$y2, $x3,$y3)=triangle_normalize($x1,$y1, $x2,$y2, $x3,$y3);
      ### normalized: "$x1,$y1, $x2,$y2, $x3,$y3"

      printwrap("($x1,$y1),");
      $TurnRuns2++;
    }
  }
  printwrap("}");
  printwrap("{ \\p pic [transform shape] {my $side triangle} };\n");
}

my $A = A($level);
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyA}{$A}\n";
print "  \\newcommand{\\MyTurnTwo}{$TurnRuns2}\n";
print "% GP-Test  A($level)==$A\n";
print "% GP-Test  TurnRuns2($level)==$TurnRuns2\n";

print "  % end generated\n";
exit 0;
