#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'sum';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 2;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi - 1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}
$Printwrap = 2;
printwrap("% N=$n_lo to $n_hi\n");
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\begin{scope}\n");
my %count;
foreach my $side ('right',
                  'left',
                 ) {
  $Printwrap = 3;
  my %seen;
  foreach my $n ($n_lo .. $n_hi - 1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    if ($side eq 'right') {
      $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
    } else {
      $path->_UNDOCUMENTED__n_segment_is_right_boundary($n_hi-1 - $n) or next;
      ### left boundary ...
      $x1 += $dx;
      $y1 += $dy;
      $dx = -$dx;
      $dy = -$dy;
    }
    my $x2 = $x1 + $dx;
    my $y2 = $y1 + $dy;

    ($dx,$dy) = xy_rotate_minus120($dx,$dy);
    ### rotated: "$dx,$dy"
    my $x3 = $x2 + $dx;
    my $y3 = $y2 + $dy;

    if ($y1 != $y2) {
      ($x1,$y1, $x2,$y2, $x3,$y3) = ($x2,$y2, $x3,$y3, $x1,$y1);
    }
    if ($x1 > $x2) {
      ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
    }
    ### triangle: "$x1,$y1 to $x2,$y2 to $x3,$y3"

    if ($seen{"$x1,$y1"}++) {
      ### duplicate ...
      next;
    }
    my $yplus = ($y1 < $y3 ? '+' : '-');
    my $yminus = ($y1 < $y3 ? '-' : '+');
    $Printwrap = 4;
    printwrap("\\fill ($x1+3*\\MyMargin,$y1$yplus\\MyMargin)");
    $Printwrap += 2;
    printwrap(" --($x2-3*\\MyMargin,$y2$yplus\\MyMargin)");
    printwrap(" --($x3,{$y3$yminus\\MyMargin*sqrt(3)})--cycle;\n");
    $count{$side}++;
  }
}
printwrap_newline();
$Printwrap = 2;
my $count_total = sum(0, values %count);
printwrap("\\end{scope}\n");

print "  \\newcommand{\\MyRT}{$count{right}}\n";
print "  \\newcommand{\\MyLT}{$count{left}}\n";
print "  \\newcommand{\\MyBT}{$count_total}\n";
print "% GP-DEFINE  gen_boundary_triangles_RT = $count{right};\n";
print "% GP-DEFINE  gen_boundary_triangles_LT = $count{left};\n";
print "% GP-DEFINE  gen_boundary_triangles_BT = $count_total;\n";
print "% GP-Test-Last  RT($level)==gen_boundary_triangles_RT\n";
print "% GP-Test-Last  LT($level)==gen_boundary_triangles_LT\n";
print "% GP-Test-Last  BT($level)==gen_boundary_triangles_BT\n";
printwrap("% end generated\n");
exit 0;
