#!/usr/bin/perl -w

# Copyright 2014, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::BaseCnv 'cnv';
use Math::PlanePath::TerdragonCurve;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level_limit = 4;
print "  % generated by tools/$FindBin::Script\n";

sub is_cantor {
  my ($n) = @_;
  my $n3 = Math::BaseCnv::cnv($n,10,3);
  return ($n3 !~ /1/);
}

foreach my $level (0 .. $level_limit) {
  my $scale = 3**($level_limit-$level);
  my $power = 3**$level;

  my $segments = MyLineSegments->new;
  foreach my $n (0 .. $power-1) {
    if (is_cantor($n)) {
      $segments->add($n,0, $n+1,0);
    }
  }

  $Printwrap = 2;
  printwrap("\\draw [yshift=$level*\\MyYshift]");
  $Printwrap = 4;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
