#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

use Math::PlanePath;
*_divrem_mutate = \&Math::PlanePath::_divrem_mutate;

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";

my $n_hi = 53;

sub n_to_encRL_by_digits_low {
  my ($n) = @_;
  ### n_to_encRL_by_digits_low(): $n

  {
    # strip all low 2s
    my $low;
    do {
      $low = _divrem_mutate($n,3);
    } while ($low == 2);
    ### $low
    ### $n

    my $prev = $low;
    if ($low == 0) {
      ### right 20 non 11,12 ...
      while ($n) {
        my $digit = _divrem_mutate($n,3);
        if ($digit == 1 && $prev != 0) { return 0; }
        if ($digit == 2 && $prev == 0) { return 1; }
        $prev = $digit;
      }
    } else {
      ### left 11,10 non 02 ...
      while ($n) {
        my $digit = _divrem_mutate($n,3);
        if ($digit == 1 && $prev != 2) { return -1; }
        if ($digit == 0 && $prev == 2) { return 0; }
        $prev = $digit;
      }
    }
    return 0;
  }
}

my %want_RL = (right => 1, left => -1);
my $path = Math::PlanePath::TerdragonCurve->new;
foreach my $side ('right', 'left') {

  print "  \\begin{scope}[shift=(pos-$side)]\n";
  print "    \\node (start-$side) at (0,0) [my dot];\n";
  $Printwrap=4;
  printwrap("\\draw [my thin] (start-$side)");
  $Printwrap=6;

  foreach my $n (1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    {
      # before $n
      my ($dx,$dy) = $path->n_to_dxdy($n-1);
      printwrap("--($x-\\MyChamfer*$dx,{($y-\\MyChamfer*$dy)*sqrt(3)})");
    }
    {
      # after $n
      my ($dx,$dy) = $path->n_to_dxdy($n);
      printwrap("--($x+\\MyChamfer*$dx,{($y+\\MyChamfer*$dy)*sqrt(3)})");
    }
  }
  {
    # part-way into segment after $n_hi
    my $n = $n_hi;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    printwrap("--($x+\\MyEnd*$dx,{($y+\\MyEnd*$dy)*sqrt(3)})");
  }
  printwrap(";\n");

  $Printwrap=4;
  printwrap("\\draw [my thick]");
  $Printwrap=6;
  foreach my $n (0 .. $n_hi-1) {
    n_to_encRL_by_digits_low($n) == $want_RL{$side} || next;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    printwrap(" ($x+\\MyEnc*$dx,{($y+\\MyEnc*$dy)*sqrt(3)})");

    $x += $dx;
    $y += $dy;
    printwrap("--($x-\\MyEnc*$dx,{($y-\\MyEnc*$dy)*sqrt(3)})");
  }
  printwrap(";\n");

  print "  \\end{scope}\n";
}
print "  % end generated\n";
exit 0;








__END__

# print "  % generated by tools/$FindBin::Script\n";
#
# my $path = Math::PlanePath::TerdragonCurve->new;
#
# my $level = 4;
# print "  \\newcommand{\\MyLevel}{$level}\n";
#
# my ($n_lo, $n_hi) = $path->level_to_n_range($level);
#
# my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
# print "  \\node (start) at (0,0) [my dot];\n";
# print "  \\node (end) at ($x_hi,$y_hi) [my dot];\n";
#
# sub n_to_enc_by_digits_low {
#   my ($n) = @_;
#   ### n_to_enc_by_digits_low(): $n
#
#   # strip all low 2s
#   my $low;
#   do {
#     $low = _divrem_mutate($n,3);
#   } while ($low == 2);
#
#   my $prev = $low;
#   while ($n) {
#     my $digit = _divrem_mutate($n,3);
#     if ($low == 1) {
#       $digit = 2 - $digit;  # flip 0<->2
#     }
#     # 21,11 -> non,   02 -> enc
#     # 01,11 -> enc,   20 -> non   flipped
#     # is high to low 12,11 and 20
#     if ($digit == 1 && $prev != 0) { return $low; }
#     if ($digit == 2 && $prev == 0) { return 1-$low; }
#     $prev = $digit;
#   }
#   return 0;
# }
#
# # non-enclosure
# foreach my $want_enc (0, 1) {
#   my $segments = MyLineSegments->new;
#   foreach my $n ($n_lo .. $n_hi-1) {
#     my ($x,$y) = $path->n_to_xy($n);
#     next unless n_to_enc_by_digits_low($n) == $want_enc;
#     my ($dx,$dy) = $path->n_to_dxdy($n);
#     $segments->add($x,$y, $x+$dx,$y+$dy,0);
#   }
#
#   $Printwrap = 2;
#   if ($want_enc) {
#     printwrap("\\draw [my enclosure line]");
#   } else {
#     printwrap("\\draw [my line]");
#   }
#   $Printwrap = 3;
#   foreach my $line ($segments->lines) {
#     my ($x1,$y1, $x2,$y2) = @$line;
#     printwrap(" ($x1,$y1)--($x2,$y2)");
#   }
#   printwrap(";\n");
# }
#
#
# print "  % end generated\n";
# exit 0;
