#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';
use MyLineSegments;

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";

my $segments = MyLineSegments->new;
$segments->add(-2,0, 2,0);
$segments->add(-1,-1, 1,1);
$segments->add(1,-1, -1,1);

$segments->add(2,0, 1,1);
$segments->add(1,1, -1,1);
$segments->add(-1,1, -2,0);
$segments->add(-2,0, -1,-1);
$segments->add(-1,-1, 1,-1);
$segments->add(1,-1, 2,0);

$Printwrap = 2;
printwrap("\\filldraw (0,0) circle;\n");
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

foreach my $level (1 .. 2) {
  my $new_segs = MyLineSegments->new;
  foreach my $line ($segments->segments) {
    my ($x1,$y1, $x2,$y2) = @$line;
    ### segment: "$x1,$y1 to $x2,$y2"

    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    # (x+iy)*(1+w) = (x-2y) + (2x+y)i
    ($x1,$y1) = mul($x1,$y1);
    ### multiply to: "$x1,$y1  dxdy=$dx,$dy"

    $new_segs->add($x1,$y1, $x1 + $dx, $y1 + $dy);
    $x1 += $dx;
    $y1 += $dy;

    {
      my ($rx,$ry) = xy_rotate_plus120($dx,$dy);
      $new_segs->add($x1,$y1, $x1 + $rx, $y1 + $ry);
      $x1 += $rx; $y1 += $ry;
    }

    $new_segs->add($x1,$y1, $x1 + $dx, $y1 + $dy);
  }
  $segments = $new_segs;

  printwrap_indent("");
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(right-pos-$level)]\n");
  $Printwrap = 4;
  printwrap("\\filldraw (0,0) circle;\n");
  printwrap("\\draw ");
  $Printwrap = 6;
  $segments->printwrap;
  printwrap(";\n");
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
