#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

sub transform {
  my ($x,$y) = @_;
  my $k = $level;
  while ($k > 3) {
    $level -= 2;
    ($x,$y) = xy_rotate_minus60($x,$y);
  }
  if ($k == 3) {
    ($x,$y) = ($y,-$x);  # rotate -90
    $k -= 3;
  }
  $k==0 or die;
  return ($x,$y);
}
sub stringize {
  my ($x,$y) = @_;
  if (($level&1)==0) {
    if ($y) { $y = "{$y*sqrt(3)}"; }
  } else {
    if ($x) { $x = "{$x*sqrt(3)}"; }
  }
  return "($x,$y)";
}

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap = 4;
$segments->printwrap(transform => \&transform,
                     stringize => \&stringize);
printwrap(";\n");

print "  % end generated\n";
exit 0;
