#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use FindBin;
use List::Util 'min';
use Math::PlanePath::TerdragonCurve 118; # v.118 for xyxy_to_n_either()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon '@dir6_to_dx','@dir6_to_dy','xy_rotate_minus60';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;
$Printwrap = 2;
printwrap("% generated by tools/$FindBin::Script\n");

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
printwrap("% level=$level N=$n_lo to $n_hi\n");

sub transform {
  my ($x,$y) = @_;
  return xy_rotate_minus60($x,$y);
}

foreach my $part ('level','infinite','all') {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$part)]\n");
  $Printwrap = 4;

  {
    my @n_list = (0);
    foreach my $k ($part eq 'level' ? ($level)
                   : $part eq 'infinite' ? ()
                   : (1 .. $level)) {
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      push @n_list, $n_hi;
    }
    foreach my $n (@n_list) {
      my ($x,$y) = $path->n_to_xy($n);
      ($x,$y) = transform($x,$y);
      printwrap(" \\node at ($x,$y) [my dot];");
    }
    printwrap_newline();
  }

  my $n_top = ($part eq 'level' ? $n_hi - 1 : $n_hi);

  my $thin = MyLineSegments->new;
  my $thick = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_top) {
    my $is_boundary;
    if ($part eq 'infinite') {
      $is_boundary = $path->_UNDOCUMENTED__n_segment_is_left_boundary($n);
    } elsif ($part eq 'level') {
      $is_boundary = $path->_UNDOCUMENTED__n_segment_is_left_boundary($n,$level);
    } else {
      $is_boundary = $path->_UNDOCUMENTED__n_segment_is_any_left_boundary($n);
    }
    my $segments = ($is_boundary ? $thick : $thin);
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }

  my @n_extra_dashed;

  # thick continuation
  if ($part eq 'infinite' || $part eq 'all') {
    my $count = 0;
    for (my $n = $n_top; ; $n++) {
      if ($path->_UNDOCUMENTED__n_segment_is_left_boundary($n)) {
        $thick->add($path->n_to_xy($n),
                    $path->n_to_xy($n+1));
        last if ++$count == 5;
        push @n_extra_dashed, $n+1;
      }
    }
  }

  foreach my $thickness ('thick','thin') {
    my $segments = ($thickness eq 'thick' ? $thick : $thin);
    $Printwrap = 4;
    printwrap("\\draw[my $thickness] ");
    $Printwrap = 6;
    $segments->printwrap(transform => \&transform);
    printwrap(";\n");
  }
  printwrap_newline();

  if ($part eq 'infinite' || $part eq 'all') {
    $Printwrap = 4;
    printwrap("\\draw[my thin]");
    $Printwrap = 6;
    foreach my $n ($n_lo .. $n_top, @n_extra_dashed) {
      my ($x,$y) = $path->n_to_xy($n);
      foreach my $dir (0 .. $#dir6_to_dx) {
        my $dx = $dir6_to_dx[$dir];
        my $dy = $dir6_to_dy[$dir];
        my $x2 = $x + $dx;
        my $y2 = $y + $dy;

        my $n_seg = $path->xyxy_to_n_either($x,$y, $x2,$y2) // next;
        next if $n_seg < $n_top;

        $x2 = $x + $dx*.75;
        $y2 = $y + $dy*.75;
        my ($x1,$y1) = transform($x,$y);
        ($x2,$y2) = transform($x2,$y2);
        printwrap(" ($x1,$y1)--($x2,$y2)");
      }
    }
    printwrap(";\n");
    printwrap_newline();
  }

  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}
printwrap("% end generated\n");
exit 0;
