#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'dxdy_to_dir6','xy_to_node_name';
use MyPrintwrap ':all';


my $level = 4;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

$Printwrap = 2;
my @num_to_type = (undef,
                   'my single',
                   'my double',
                   'my triple');
my @count = (undef,0,0,0);
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;
  next unless $n == $n_list[0];

  my $num = scalar(@n_list);
  my $type = $num_to_type[$num];
  my $name = xy_to_node_name($x,$y);
  printwrap(" \\node ($name) at ($x,{$y*sqrt(3)}) [$type];");
  $count[$num]++;
}
printwrap_newline();

sub dxdy_to_dir360 {
  my ($dx,$dy) = @_;
  return dxdy_to_dir6($dx,$dy) * 60;
}

$Printwrap = 2;
printwrap("\\draw [thin] ");
$Printwrap = 4;
my $join = '';
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my $x2 = $x1 + $dx;
  my $y2 = $y1 + $dy;
  my $p1 = xy_to_node_name($x1,$y1);
  my $p2 = xy_to_node_name($x2,$y2);
  my $a1 = dxdy_to_dir360($dx,$dy);
  my $a2 = ($a1 + 180) % 360;
  printwrap("(\$($p1.$a1)!\\MyFrac!($p2.$a2)\$)--(\$($p2.$a2)!\\MyFrac!($p1.$a1)\$)");
  $join = '--';
}
printwrap(";\n");

my $count_total = $count[1] + $count[2] + $count[3];
print "  \\newcommand{\\MyS}{$count[1]}  % singles\n";
print "  \\newcommand{\\MyD}{$count[2]}  % doubles\n";
print "  \\newcommand{\\MyT}{$count[3]}  % triples\n";
print "  \\newcommand{\\MyP}{$count_total}  % total\n";
print "% GP-Test-Last  S($level)==$count[1]\n";
print "% GP-Test-Last  D($level)==$count[2]\n";
print "% GP-Test-Last  T($level)==$count[3]\n";
print "% GP-Test-Last  P($level)==$count_total\n";
print "  % end generated\n";
exit 0;
