#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'xy_to_node_name';
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my @dir4_to_dx = (1,0,-1,0);
my @dir4_to_dy = (0,1,0,-1);

$Printwrap = 2;
my %nodes;
my @edges;
my $limit = 2**$level-1;
foreach my $x (1 .. $limit) {
  foreach my $y (0 .. $x-1) {
    next if $x & $y;
    next if $x + $y > $limit;
    my $name = xy_to_node_name($x,$y);
    printwrap(" \\node ($name) at ($x,$y) [my dot];");
    $nodes{$name} = 1;
    foreach my $dir4 (0 .. 3) {
      my $x2 = $x + $dir4_to_dx[$dir4];
      my $y2 = $y + $dir4_to_dy[$dir4];
      my $name2 = xy_to_node_name($x2,$y2);
      if ($nodes{$name2}) {
        push @edges, " ($name2)--($name)";
      }
    }
  }
}

$Printwrap = 2;
printwrap("\\draw[my edge]");
$Printwrap += 2;
foreach my $str (@edges) {
  printwrap($str);
}
printwrap(";\n");

print "% GP-Test-Last  TTV($level) == ",scalar(keys %nodes),"\n";
print "  % end generated\n";
exit 0;
