#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon '@dir3_to_dx','@dir3_to_dy','xy_rotate_plus60';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

$Printwrap = 2;
my $segments = MyLineSegments->new;
foreach my $elem ([0,0, 0],
                  [0,0, 1],
                  [3,1, 0],
                  [3,1, 1],
                  [3,1, 2],
                  [9,1, 1],
                  [9,-1, 1],
                  [6,0, 0],
                  [6,0, 1],
                  [6,2, 2],
                  [3,-1, 0],
                  [3,-1, 1],
                  [0,2, 2],
                  [-3,1, 0],
                  [-3,1, 2],
                  [-3,-1, 0],
                 ) {
  my ($x,$y, $dir3) = @$elem;
  my $rotate = $dir3 * 120;
  printwrap (" \\pic at ($x/2,{$y*sqrt(3)/2}) [transform shape,rotate=$rotate] {my rhombus pic};");
  my $dx = $dir3_to_dx[$dir3];
  my $dy = $dir3_to_dy[$dir3];
  my ($lx,$ly) = xy_rotate_plus60($dx,$dy);
  $segments->add($x,$y, $x+$dx,$y+$dy);
  $segments->add($x,$y, $x+$lx,$y+$ly);
  $segments->add($x+$dx,$y+$dy, $x+$dx+$lx,$y+$dy+$ly);
  $segments->add($x+$lx,$y+$ly, $x+$dx+$lx,$y+$dy+$ly);
}
printwrap("\n");

print "  \\begin{scope}[xscale=1/2,yscale={sqrt(3)/2}]\n";

$Printwrap = 2;
printwrap("\\draw[my line] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
