#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Usage: perl test.pl
#
# Run some self-tests on ../terdragon.tex and on MyTerdragon.pm.
#

use 5.004;
use strict;
use File::Slurp;
use IPC::Run;
use List::Util 'max','min';
use Math::PlanePath::TerdragonCurve;
use Math::Geometry::Planar;

use FindBin;
use lib::abs $FindBin::Bin;
use MyTerdragon ':all';


my $source_dir = File::Spec->catdir ($FindBin::Bin, File::Spec->updir);
$source_dir = File::Spec->rel2abs($source_dir);
print "source dir $source_dir\n";

chdir $source_dir or die $!;

my $terdragon_tex_filename = 'terdragon.tex';
# $terdragon_tex_filename = 'devel/more.tex';
my $terdragon_pdf_filename = 'terdragon.pdf';
my $terdragon_tex_str = File::Slurp::slurp($terdragon_tex_filename);
print count_lines($terdragon_tex_str)," lines\n";

# my $terdragon_tex_str_sans_comments = $terdragon_tex_str;
# $terdragon_tex_str_sans_comments =~ s/((^|[^\\])(\\\\)*)%[^\n]*/$1/mg;
# print count_lines($terdragon_tex_str_sans_comments)," lines\n";


#------------------------------------------------------------------------------
# ../devel/terdragon-defines.gp
# ../devel/terdragon-defines.header.gp

{
  # same copyright years as terdragon.tex where defines are extracted from
  $terdragon_tex_str =~ /^% (Copyright .*)/m
    or die "terdragon.tex copyright line not found";
  my $want_copyright = $1;
  foreach my $filename ('devel/terdragon-defines.gp',
                        'devel/terdragon-defines.header.gp') {
    my $str = File::Slurp::slurp($filename);
    $str =~ /^\\\\ \Q$want_copyright\E$/m
      or die "$filename copyright different from terdragon.tex";
  }
}

#------------------------------------------------------------------------------
# terdragon.pdf Title and Author

{
  IPC::Run::run(['pdfinfo', '-isodates', $terdragon_pdf_filename],
                '>', \my $pdfinfo_out) or die;
  $pdfinfo_out =~ s/^Jav.*\n//m;
  print $pdfinfo_out;
  {
    $pdfinfo_out =~ /^Title:\s*(.*)/m
      or die;
    my $title = $1;
    # print "title: $title\n";
    $title =~ /Terdragon/
      or die;
  }
  {
    $pdfinfo_out =~ /^Author:\s*(.*)/m
      or die;
    my $author = $1;
    # print "author: $author\n";
    $author =~ /Kevin/
      or die;
  }
}

#------------------------------------------------------------------------------
sub pos_linenum {
  my ($str) = @_;
  $str = substr($str, 0, pos($_[0]));
  return 1 + scalar($str =~ tr/\n//);
}

sub count_lines {
  my ($str) = @_;
  my $count = ($str =~ tr/\n/\n/);
  if ($str =~ /[^\n]$/) { $count++; }
  return $count;
}

#------------------------------------------------------------------------------
exit 0;
