#!/usr/bin/perl -w

# Copyright 2014 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


printwrap("  % generated by tools/$FindBin::Script\n");

my $level = 4;

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

printwrap_indent("  ");
printwrap("\\begin{scope}[rotate={-atan2($y_hi*sqrt(3),$x_hi)},\n");
printwrap("              scale={1/veclen($x_hi,$y_hi*sqrt(3))},\n");
printwrap("              yscale=sqrt(3),\n");
printwrap("              ultra thin]\n");
printwrap_indent("    ");
printwrap("\\draw");
printwrap_indent("     ");

# {
#   printwrap_indent("     ");
#
#   my $join = '';
#   foreach my $n ($n_lo .. $n_hi) {
#     my $end = ($n == $n_hi ? ';' : '');
#     my ($x,$y) = $path->n_to_xy($n);
#     printwrap("$join($x,$y)$end");
#     $join = '--';
#   }
# }

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }
  ### $segments

  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");
}

printwrap_indent("  ");
printwrap("\\end{scope}\n");
printwrap("% end generated\n");
exit 0;
